@@ -17,6 +17,7 @@
 
 #include <config.h>
 
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -29,6 +30,7 @@
 #include <unitypes.h>
 #include <uniconv.h>
 #include <unistr.h>
+#include <unistring/localcharset.h>
 
 /* Gnulib headers. */
 #include "error.h"
@@ -36,12 +38,14 @@
 #define _(String) dgettext (PACKAGE, String)
 #include "progname.h"
 #include "version-etc.h"
-#include "localcharset.h"
 
 #include "idn2_cmd.h"
 
 #include "blurbs.h"
 
+#ifdef __cplusplus
+extern // define a global const variable in C++, C doesn't need it.
+#endif
 const char version_etc_copyright[] =
   /* Do *not* mark this string for translation.  %s is a copyright
      symbol suitable for this locale, and %d is the copyright
@@ -78,14 +82,17 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -V, --version            Print version and exit\n\
 "), stdout);
       fputs (_("\
+  -d, --decode             Decode (punycode) domain name\n\
   -l, --lookup             Lookup domain name (default)\n\
   -r, --register           Register label\n\
 "), stdout);
       fputs (_("\
   -T, --tr46t              Enable TR46 transitional processing\n\
   -N, --tr46nt             Enable TR46 non-transitional processing\n\
+      --no-tr46            Disable TR46 processing\n\
 "), stdout);
       fputs (_("\
+      --usestd3asciirules  Enable STD3 ASCII rules\n\
       --debug              Print debugging information\n\
       --quiet              Silent operation\n\
 "), stdout);
@@ -98,11 +105,12 @@ static void
 hexdump (const char *prefix, const char *str)
 {
   uint8_t *u8;
-  uint32_t *u32;
+  uint32_t *u32 = NULL;
   size_t u32len;
   size_t i;
+  const char *encoding = locale_charset ();
 
-  u8 = u8_strconv_from_locale (str);
+  u8 = u8_strconv_from_encoding (str, encoding, iconveh_error);
   if (u8)
     u32 = u8_to_u32 (u8, strlen ((char *) u8), NULL, &u32len);
 
@@ -128,6 +136,7 @@ process_input (char *readbuf, int flags)
 {
   size_t len = strlen (readbuf);
   char *output;
+  const char *tag;
   int rc;
 
   if (len && readbuf[len - 1] == '\n')
@@ -148,9 +157,20 @@ process_input (char *readbuf, int flags)
     hexdump ("input", readbuf);
 
   if (args_info.register_given)
-    rc = idn2_register_ul (readbuf, NULL, &output, flags);
+    {
+      rc = idn2_register_ul(readbuf, NULL, &output, flags);
+      tag = "register";
+    }
+  else if (args_info.decode_given)
+    {
+      rc = idn2_to_unicode_lzlz (readbuf, &output, 0);
+      tag = "decode";
+    }
   else
-    rc = idn2_lookup_ul (readbuf, &output, flags);
+    {
+      rc = idn2_to_ascii_lz (readbuf, &output, flags);
+      tag = "toAscii";
+    }
 
   if (rc == IDN2_OK)
     {
@@ -161,16 +181,14 @@ process_input (char *readbuf, int flags)
       free (output);
     }
   else
-    error (EXIT_FAILURE, 0, "%s: %s",
-	   args_info.register_given ? "register" : "lookup",
-	   idn2_strerror (rc));
+    error (EXIT_FAILURE, 0, "%s: %s", tag, idn2_strerror (rc));
 }
 
 int
 main (int argc, char *argv[])
 {
   unsigned cmdn;
-  int flags = 0;
+  int flags = IDN2_NONTRANSITIONAL;
 
   setlocale (LC_ALL, "");
   set_program_name (argv[0]);
@@ -206,6 +224,11 @@ main (int argc, char *argv[])
     flags = IDN2_TRANSITIONAL;
   else if (args_info.tr46nt_given)
     flags = IDN2_NONTRANSITIONAL;
+  else if (args_info.no_tr46_given)
+    flags = 0;
+
+  if (flags && args_info.usestd3asciirules_given)
+    flags |= IDN2_USE_STD3_ASCII_RULES;
 
   for (cmdn = 0; cmdn < args_info.inputs_num; cmdn++)
     process_input (args_info.inputs[cmdn], flags | IDN2_NFC_INPUT);
